'use strict';Object.defineProperty(exports, '__esModule', { value: true });var _createClass = (function () {function defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ('value' in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}return function (Constructor, protoProps, staticProps) {if (protoProps) defineProperties(Constructor.prototype, protoProps);if (staticProps) defineProperties(Constructor, staticProps);return Constructor;};})();function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { 'default': obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError('Cannot call a class as a function');}}var _jsYaml = require('js-yaml');var _jsYaml2 = _interopRequireDefault(_jsYaml);var _fs = require(
'fs');var _fs2 = _interopRequireDefault(_fs);var _path = require(
'path');var _path2 = _interopRequireDefault(_path);var _rimraf = require(
'rimraf');var _rimraf2 = _interopRequireDefault(_rimraf);var _ComponentFinderJs = require(

'../ComponentFinder.js');var _ComponentFinderJs2 = _interopRequireDefault(_ComponentFinderJs);var _generateServerFileJs = require(
'./generateServerFile.js');var _generateServerFileJs2 = _interopRequireDefault(_generateServerFileJs);var _generateListPageJs = require(
'./generateListPage.js');var _generateListPageJs2 = _interopRequireDefault(_generateListPageJs);

var BUILD_RELATIVE_DIR = 'build';
var PRECOMPILE_RELATIVE_DIR = 'build/precompile';
var SERVER_FILE = 'server.js';
var CMPT_BROWSER_LIST_PAGE = 'CmptBrowserListPage.js';

/**
 * Does pre-compilation routine:
 *
 * 1. Generates list of pages defined in the routes file.
 * 2. Generates list of components for component browser.
 * 3. Generates server.js file that imports all pages defined in routes file.
 * 4. Generates component browser search page.
 * 5. Generates component browser detail page.
 *
 * Step number 2, 4, and 5 are only executed when componentBrowser is on.
 *
 * @SERVER
 */var 
Precompiler = (function () {


  function Precompiler(frameworkConfig) {_classCallCheck(this, Precompiler);
    this._frameworkConfig = frameworkConfig;}_createClass(Precompiler, [{ key: 'precompile', value: 


    function precompile() {
      var pages = this.generatePageList();
      var components = this.generateComponentList();
      var buildDir = Precompiler.getBuildDir(this._frameworkConfig);
      var precompileDir = Precompiler.getPrecompileDir(this._frameworkConfig);
      Precompiler.cleanDir(buildDir);
      Precompiler.cleanDir(precompileDir);

      if (this._frameworkConfig.componentBrowser) {
        var listFilePath = Precompiler.getCmptBrowserListPagePath(this._frameworkConfig);
        _fs2['default'].writeFileSync(listFilePath, (0, _generateListPageJs2['default'])(components));

        // Add component browser pages to list of pages to require.
        pages.push(Precompiler.getCmptBrowserListPageRelativePath());}


      // Generate server.js file, containing references to all pages needed for
      // server side routing, plus all component definition for component browser.
      var serverFilePath = Precompiler.getServerFilePath(this._frameworkConfig);
      _fs2['default'].writeFileSync(serverFilePath, (0, _generateServerFileJs2['default'])(pages, components));}


    /**
     * Returns an array containing relative page paths:
     *
     * <pre>
     *   [
     *     'src/../../../Page.js',
     *     ...
     *   ]
     * </pre>
     *
     * @return {Array}
     */ }, { key: 'generatePageList', value: 
    function generatePageList() {
      var routeId, routes = Precompiler.readRoutesFile(this._frameworkConfig);
      var pageList = [];
      for (routeId in routes) {
        // TODO: Check if file exists or not, throw easy to read error when it doesn't exist.
        pageList.push(routes[routeId].page);}

      return pageList;}


    /**
     * Returns an array containing component information:
     * 
     * <pre>
     *   {
     *     vendorName: {
     *       componentName: {
     *         
     *       }
     *     }
     *   }
     * </pre>
     * 
     * @returns {Object}
     */ }, { key: 'generateComponentList', value: 
    function generateComponentList() {var _this = this;
      var components = {};
      var componentFinder = new _ComponentFinderJs2['default']();
      // TODO Multiple component source directory.
      var sourceDirectory = _path2['default'].join(this._frameworkConfig.absoluteProjectDir, 'src');
      componentFinder.find(sourceDirectory, function (thumbRelativePath, testRelativePath, docRelativePath, code, componentJson) {
        if (!components.hasOwnProperty(componentJson.vendor)) {
          components[componentJson.vendor] = {};}

        if (components[componentJson.vendor].hasOwnProperty(componentJson.name)) {
          console.log('Duplicate component, will be overwritten: ' + componentJson.vendor + '.' + componentJson.name);}


        thumbRelativePath = _this._prepWithSrc(thumbRelativePath);
        testRelativePath = _this._prepWithSrc(testRelativePath);
        docRelativePath = _this._prepWithSrc(docRelativePath);
        components[componentJson.vendor][componentJson.name] = { 
          thumbnail: thumbRelativePath, 
          test: testRelativePath, 
          doc: docRelativePath, 
          detail: componentJson, 
          code: code };});


      return components;} }, { key: '_prepWithSrc', value: 


    function _prepWithSrc(relativePath) {
      return relativePath == null ? null : _path2['default'].join('src', relativePath);} }], [{ key: 'getServerFilePath', value: 


    function getServerFilePath(frameworkConfig) {
      return _path2['default'].join(frameworkConfig.absoluteProjectDir, PRECOMPILE_RELATIVE_DIR, SERVER_FILE);} }, { key: 'getBuildDir', value: 


    function getBuildDir(frameworkConfig) {
      return _path2['default'].join(frameworkConfig.absoluteProjectDir, BUILD_RELATIVE_DIR);} }, { key: 'getPrecompileDir', value: 


    function getPrecompileDir(frameworkConfig) {
      return _path2['default'].join(frameworkConfig.absoluteProjectDir, PRECOMPILE_RELATIVE_DIR);} }, { key: 'getCmptBrowserListPagePath', value: 


    function getCmptBrowserListPagePath(frameworkConfig) {
      return _path2['default'].join(frameworkConfig.absoluteProjectDir, PRECOMPILE_RELATIVE_DIR, CMPT_BROWSER_LIST_PAGE);} }, { key: 'getCmptBrowserListPageRelativePath', value: 


    function getCmptBrowserListPageRelativePath() {
      return _path2['default'].join(PRECOMPILE_RELATIVE_DIR, CMPT_BROWSER_LIST_PAGE);} }, { key: 'cleanDir', value: 


    function cleanDir(directory) {
      _rimraf2['default'].sync(directory);
      // Ensure directory exists.
      try {
        _fs2['default'].mkdirSync(directory);} 
      catch (e) {
        if (e.code != 'EEXIST') throw e;}}



    /**
     * @param {Object} frameworkConfig
     * @return {Object}
     */ }, { key: 'readRoutesFile', value: 
    function readRoutesFile(frameworkConfig) {
      var routesFilePath = _path2['default'].join(frameworkConfig.absoluteProjectDir, 'routes.yml');
      return _jsYaml2['default'].safeLoad(_fs2['default'].readFileSync(routesFilePath, 'utf8'));} }]);return Precompiler;})();exports['default'] = Precompiler;module.exports = exports['default'];
//# sourceMappingURL=Precompiler.js.map